------------------------------------------------------------------------------
-- SellableItemDrops - Logging your sellable item drops
------------------------------------------------------------------------------
-- Modules/GUI_inventory.lua - Interface/Inventory Tab
--
-- Author: Expelliarm5s / January 2020 / All Rights Reserved
--
-- Version 1.4.18
------------------------------------------------------------------------------
local addonName, addon = ...
local GUI = addon:GetModule("GUI")
local L = LibStub("AceLocale-3.0"):GetLocale(addonName)
local AceGUI = LibStub("AceGUI-3.0")
local private = {}
------------------------------------------------------------------------------

GUI.guiTabs.texts["inventory"] = {
	title=L["Bags"], desc=L["Bags"]}

GUI.guiTabs.order[#GUI.guiTabs.order + 1] = "inventory"

GUI.guiTabs.select["inventory"] = function(container, group)
	GUI:DebugPrintf("Select Tab %s: %s", group, tostring(container))
	if not container then
		GUI:DebugPrintf("Select Tab: container is nil")
	end

	-- min. Size
	GUI.display.frame:SetMinResize(400, 400)

	-- load items from bags
	addon.invInfo = {}
	addon.invItems = addon.Data:LoadFromBags({})

	-- Filter: default filter is empty
	GUI.invFilters = {}

	-- variants
	local flNotTheRealItemString = ""
	if (addon.db.global.useNotTheRealItemString) then
		flNotTheRealItemString = ITEM_QUALITY_COLORS[5].hex .. L[" (! = variant)"] .. FONT_COLOR_CODE_CLOSE
	end

	if (CanIMogIt) then
		flNotTheRealItemString = flNotTheRealItemString .. " " .. ITEM_QUALITY_COLORS[5].hex .. L[" (TM = New Transmog)"] .. FONT_COLOR_CODE_CLOSE
	end

	-- bag summary group
	local grp1 = AceGUI:Create("SIDInlineGroup")
	grp1:SetLayout("Flow")
	if grp1.SetTitle then
		grp1:SetTitle(L["Bag Summary"])
	end
	grp1:SetFullWidth(true)

	local grpWidth
	local grpColNum
	local grpRelWidth

	-- bag summary group - line 1
	GUI.lblInv = {}
	GUI.lblInv.liv = GUI.AddButton("Label", L["LIV"], "-")
	GUI.lblInv.liv:SetRelativeWidth(0.50)
	grp1:AddChild(GUI.lblInv.liv)

	GUI.lblInv.source = GUI.AddButton("Label", L["Price Source"], "-")
	GUI.lblInv.source:SetRelativeWidth(0.50)
	grp1:AddChild(GUI.lblInv.source)

	-- bag summary group - line 2
	GUI.infoInv = {}
	GUI.infoInv.liv = GUI.AddButton("Label", "-", "-")
	GUI.infoInv.liv:SetRelativeWidth(0.50)
	grp1:AddChild(GUI.infoInv.liv)

	GUI.infoInv.source = GUI.AddButton("Label", "-", "-")
	GUI.infoInv.source:SetRelativeWidth(0.50)
	grp1:AddChild(GUI.infoInv.source)

	container:AddChild(grp1)

	-- bag data group
	local grp2 = AceGUI:Create("SIDInlineGroup")
	grp2:SetLayout("Flow")
	if grp2.SetTitle then
		if addon.db.global.stackedView then
			grp2:SetTitle(L["Bag Data"] .. L[" (stacked view)"])
		else
			grp2:SetTitle(L["Bag Data"])
		end
	end
	grp2:SetFullWidth(true)
	grp2:SetFullHeight(true)

	local grp3 = AceGUI:Create("SimpleGroup")
	grp3:SetLayout("Flow")
	grp3:SetFullWidth(true)

	grpWidth = 1
	grpColNum = 3.01

	grpRelWidth = grpWidth / grpColNum

	-- bag data group - line 1
	local i = GUI.AddButton("Label", L["Search"], "-")
	i:SetRelativeWidth(grpRelWidth)
	grp3:AddChild(i)

	i = GUI.AddButton("Label", L["Rarity"], "-")
	i:SetRelativeWidth(grpRelWidth)
	grp3:AddChild(i)

	i = GUI.AddButton("Label", L["Bag"], "-")
	i:SetRelativeWidth(grpRelWidth)
	grp3:AddChild(i)

	grp2:AddChild(grp3)

	grp3 = AceGUI:Create("SimpleGroup")
	grp3:SetLayout("Flow")
	grp3:SetFullWidth(true)

	-- bag data group - line 2
	i = GUI.AddButton("SIDEditBox", "", "-")
	i:SetRelativeWidth(grpRelWidth)
	i.button:SetScript("OnClick", function(f)
			local editbox = f.obj.editbox
			editbox:SetText("")
			editbox:SetCursorPosition(0)
			editbox:ClearFocus()
			f.obj:DisableButton(true)
			f.obj:Fire("OnTextChanged", "")
		end)
	i.button:SetText(L["CLR"])
	i:SetCallback("OnTextChanged",
		function(f, _, value)
			value = value:trim()
			if value == "" then
				GUI.invFilters.name = nil
			else
				GUI.invFilters.name = value
				f:DisableButton(false)
			end
			GUI.dataInv:SetContent(private.GetInventoryItemData(GUI.invFilters))
			private.GetInventoryInfoData(GUI.invFilters)
		end
	)
	grp3:AddChild(i)

	i = GUI.AddButton("Dropdown", L["Rarity"], "-")
	i:SetRelativeWidth(grpRelWidth)
	i:SetList(GUI.rarityList)
	i:SetCallback("OnValueChanged",
		function(_, _, key)
			if key > 0 then
				GUI.invFilters.rarity = key
			else
				GUI.invFilters.rarity = nil
			end
			GUI.dataInv:SetContent(private.GetInventoryItemData(GUI.invFilters))
			private.GetInventoryInfoData(GUI.invFilters)
		end
	)
	grp3:AddChild(i)

	i = GUI.AddButton("Dropdown", L["Bag"], "-")
	i:SetRelativeWidth(grpRelWidth)
	i:SetList(GUI.zoneListFromItems(addon.invItems))
	local value = "all"
	i:SetText(i.list[value])
	i.value = value
	i:SetCallback("OnValueChanged",
		function(_, _, val)
			if val == "all" then
				GUI.invFilters.zone = nil
			else
				GUI.invFilters.zone = val
			end
			GUI.dataInv:SetContent(private.GetInventoryItemData(GUI.invFilters))
			private.GetInventoryInfoData(GUI.invFilters)
		end
	)
	grp3:AddChild(i)

	grp2:AddChild(grp3)

	-- bag item data
	local grp4 = AceGUI:Create("SimpleGroup")
	grp4:SetLayout("Fill")
	grp4:SetFullWidth(true)
	grp4:SetFullHeight(true)

	local titles = {
		{text=L["Item"] .. flNotTheRealItemString, relWidth=GUI.TITLEWIDTH_ITEM + 0.1},
		{text=L["Value   "], relWidth=-GUI.TITLEWIDTH_VALUE},
		{text=L["Qty "], relWidth=-GUI.TITLEWIDTH_QTY},
		{text=L["Player"], relWidth=GUI.TITLEWIDTH_PLAYER},
		{text=L["Bag"], relWidth=GUI.TITLEWIDTH_ZONE - 0.1},
		{text="", relWidth=GUI.TITLEWIDTH_TIME},
	}

	local colSpan = 0
	local colSpan2 = 6
	if (not addon.db.global.shPlayer) then
		colSpan = colSpan + GUI.TITLEWIDTH_PLAYER - GUI.TITLEWIDTH_GAP
		titles[4] = {text="", relWidth=GUI.TITLEWIDTH_GAP}
		colSpan2 = colSpan2 - 1
	end
	if (true) then
		colSpan = colSpan + GUI.TITLEWIDTH_TIME - GUI.TITLEWIDTH_GAP
		titles[6] = {text="", relWidth=GUI.TITLEWIDTH_GAP}
		colSpan2 = colSpan2 - 1
	end
	titles[1].relWidth = GUI.TITLEWIDTH_ITEM + 0.1 + colSpan / colSpan2
	titles[2].relWidth = -GUI.TITLEWIDTH_VALUE - colSpan / colSpan2
	titles[3].relWidth = -GUI.TITLEWIDTH_QTY - colSpan / colSpan2
	if titles[4].text ~= "" then
		titles[4].relWidth = GUI.TITLEWIDTH_PLAYER + colSpan / colSpan2
	end
	titles[5].relWidth = GUI.TITLEWIDTH_ZONE - 0.1 + colSpan / colSpan2
	if titles[6].text ~= "" then
		titles[6].relWidth = GUI.TITLEWIDTH_TIME + colSpan / colSpan2
	end

	container.titleStatusText = {
		L["Leftclick to sort this column"],
		L["Leftclick to sort this column"],
		L["Leftclick to sort this column"],
		L["Leftclick to sort this column"],
		L["Leftclick to sort this column"],
		L["Leftclick to sort this column"],
		}

	container.statusText = ""
	if Atr_IsTabSelected then
		-- container.statusText = L["Move to Bank with Leftclick"] .. " / " .. L["Ctrl-Rightclick to auction the item with Auctionator"]
		container.statusText = L["Ctrl-Rightclick to auction the item with Auctionator"]
	end

	local eventHandlers = {
		OnEnter = GUI.DataFrame_OnEnter,
		OnLeave = GUI.DataFrame_OnLeave,
		OnClick = function(_, data, button)
			if not data then return end
			if button == "LeftButton" and IsShiftKeyDown() then
				GUI:PrintItemStrings(data.link)
			end
			if button == "LeftButton" and not IsShiftKeyDown() then
				private.TransferItemFromBag(data.link, data.bagid, data.slotid)
			end
			if button == "RightButton" and IsControlKeyDown() then
				private.AuctionatorItem(data.link, data.bagid, data.slotid)
			end
		end,
	}

	GUI.dataInv = AceGUI:Create("SIDData")
	GUI.dataInv:DoRedraw(false)
	GUI.dataInv:SetEventHandlers(eventHandlers)
	GUI.dataInv:SetTitles(titles)
	GUI.dataInv:SetSortTitle(5)
	grp4:AddChild(GUI.dataInv)

	grp2:AddChild(grp4)
	container:AddChild(grp2)

	GUI.dataInv:SetContent(private.GetInventoryItemData(GUI.invFilters))
	GUI.dataInv:DoRedraw(true)

	private.GetInventoryInfoData(GUI.invFilters)

	return container
end

-- tranfer item to bank/guild bank
function private.TransferItemFromBag(link, bagid, slotid)
	GUI:Printf("TransferItemFromBag(%s, %d, %d)", link, bagid, slotid)

end

-- 	open Auctionator tab with item
function private.AuctionatorItem(link, bagid, slotid)
	GUI:DebugPrintf("AuctionatorItem(%s, %d, %d)", link, bagid, slotid)

	-- currently (8.3) not working due to AH changes

	-- check on Auctionator
	if not (Atr_IsTabSelected and Atr_SelectPane and Atr_ClearAll and Atr_ClickAuctionSellItemButton) then
		return
	end

	-- FIX for TSM3
	if TSMAPI and TSMAPI.GetItemValue then
		_G["AuctionFrameTab1"]:Click()
	end

	-- FIXME for TSM4: check if special handling necessary

	-- see if Atr loaded
	if not Atr_Main_Panel then
		return
	end

	-- see Atr_LoadContainerItemToSellPane()
	local SELL_TAB    = 1;
	if (not Atr_IsTabSelected(SELL_TAB)) then
		Atr_SelectPane(SELL_TAB);
	end

	if (not Atr_IsTabSelected(SELL_TAB)) then
		return
	end

	PickupContainerItem(bagid, slotid)

	local infoType = GetCursorInfo()

	if (infoType == "item") then
		addon:Printf("%s (Bag %d, Slot %d) -> Auctionator", link, bagid, slotid)

		Atr_ClearAll();
		Atr_ClickAuctionSellItemButton();
		ClearCursor();

		addon.invItems = addon.Data:LoadFromBags({})
		GUI.dataInv:SetContent(private.GetInventoryItemData(GUI.invFilters))
		GUI.dataInv:DoRedraw(true)
		private.GetInventoryInfoData(GUI.invFilters)
	end

end

-- prepare/update inventory item data
function private.GetInventoryItemData(lfilters)
	GUI:DebugPrintf("GetInventoryItemData()")
	if GUI.view ~= "inventory" or not GUI.display then
		GUI:DebugPrintf("... nothing done.")
		return
	end

	return GUI.GetItemData(addon.invItems, lfilters, true)
end

-- prepare/update inventory info data
function private.GetInventoryInfoData(filters)
	GUI:DebugPrintf("GetInventoryInfoData()")
	if GUI.view ~= "inventory" or not GUI.display then
		GUI:DebugPrintf("... nothing done.")
		return
	end

	if (addon.invInfo.invMVSource ~= addon.db.global.mvSource) then
		GUI:DebugPrintf("mvSource changed, update Data...")
		addon.Data:UpdateInventory()
	end

	local mvFilter = 0
	local itsSum = 0
	local itsFilter = 0
	local itSum = 0
	local itFilter = 0
	local itStart
	local itEnd
	local isFiltered = false

	if (not filters.name and not filters.rarity and not filters.player and not filters.time and not filters.zone) then
		GUI:DebugPrintf("... with no filters")
		-- done
	else
		GUI:DebugPrintf("... with filters")
		isFiltered = true
		itStart = 0
		itEnd = 0
		for link, data in pairs(addon.invItems) do
			if #data.drops > 0 then
				itsSum = itsSum + 1
			end
			if #data.drops > 0 and not GUI.IsItemFiltered(link, data, filters) then
				itsFilter = itsFilter + 1
				for _, record in ipairs(data.drops) do
					itSum = itSum + record.quantity
					if not GUI.IsRecordFiltered(record, filters) then
						if (itStart == 0) then
							itStart = record.time
						end
						if (record.time < itStart) then
							itStart = record.time
						end
						if (record.time > itEnd ) then
							itEnd = record.time
						end
						itFilter = itFilter + record.quantity
						local marketValue = addon.Data:GetItemValue(record.link) or 0
						mvFilter = mvFilter + (marketValue * record.quantity)
					end
				end
			end
		end
	end

	if (isFiltered or (itsSum ~= itsFilter) or (itSum ~= itFilter)) then
		if (GUI.infoInv) then
			-- LIV / Gold
			GUI.infoInv.liv:SetText(GUI:FromCopper(mvFilter))
			-- Price Source
			local mvSourcesList = addon.Item:GetPriceSources() or {}
			if mvSourcesList[addon.db.global.mvSource] then
				GUI.infoInv.source:SetText(ITEM_QUALITY_COLORS[6].hex ..
					addon.db.global.mvSource
					.. FONT_COLOR_CODE_CLOSE)
			else
				GUI.infoInv.source:SetText(ITEM_QUALITY_COLORS[0].hex ..
					"(" .. addon.db.global.mvSource .. ")"
					.. FONT_COLOR_CODE_CLOSE)
			end
		end
	else
		if (GUI.infoInv) then
			-- Looted Item Value / Gold
			GUI.infoInv.liv:SetText(GUI:FromCopper(addon.invInfo.invMV))
			-- Price Source
			local mvSourcesList = addon.Item:GetPriceSources() or {}
			if mvSourcesList[addon.db.global.mvSource] then
				GUI.infoInv.source:SetText(ITEM_QUALITY_COLORS[6].hex ..
					addon.db.global.mvSource
					.. FONT_COLOR_CODE_CLOSE)
			else
				GUI.infoInv.source:SetText(ITEM_QUALITY_COLORS[0].hex ..
					"(" .. addon.db.global.mvSource .. ")"
					.. FONT_COLOR_CODE_CLOSE)
			end
		end
	end
end

-- EOF

